/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file    wrt_plugin_export.h
 * @author  Przemyslaw Dobrowolski (p.dobrowolsk@samsung.com)
 * @version 1.0
 * @brief   Header file for plugin export API
 */
#ifndef WRT_PLUGIN_EXPORT_H
#define WRT_PLUGIN_EXPORT_H

#include <stddef.h>

/**
 * Widget handle type
 */
typedef int widget_handle_t;

/**
 * Invalid Widget handle
 */
#define INVALID_WIDGET_HANDLE   (-1)

/**
 * Parameter which should be used during policy evaluation.
 */
typedef struct ace_param_s
{
    const char *name;
    const char *value;
} ace_param_t;

/**
 * List of additional parameters which should be used during policy evaluation.
 */
typedef struct ace_param_list_s
{
    size_t count;
    ace_param_t *param;
} ace_param_list_t;

/**
 * Contains list of device capabilities. Each device capability may have
 * associated list of function params.
 */
typedef struct ace_device_cap_s
{
    size_t devcapsCount;
    const char**      dev_cap_names;
    size_t paramsCount;
    ace_param_list_t* params;
} ace_device_cap_t;

/**
 * List of device capabilities which must be check.
 */
typedef struct ace_device_capabilities_s
{
    size_t count;
    const char **device_cap;
} ace_device_capabilities_t;

/**
 * List of api features that must be checked
 */
typedef struct ace_api_features_s
{
    size_t count;
    const char **api_feature;
} ace_api_features_t;

/**
 * Data from request will be used to evaluate policy file.
 */
typedef struct ace_request_s
{
    widget_handle_t widget_handle;
    const char*                 feature_api;
    const char*                 function_name;
    ace_device_capabilities_t device_capabilities;
    ace_param_list_t param_list;
} ace_request_t;

/**
 * Data from request will be used to evaluate policy file.
 */
typedef struct ace_request_2_s
{
    widget_handle_t widget_handle;
    ace_api_features_t api_features;
    const char*                 function_name;
    ace_device_cap_t device_capabilities;
} ace_request_2_t;

/**
 * info returned by plugin_api_check_access
 */
#define PLUGIN_API_ACCESS_GRANTED 1
#define PLUGIN_API_ACCESS_DENIED 0
#define PLUGIN_API_ACCESS_ERROR -1


typedef const void* java_script_context_t;

typedef struct js_object_properties_s
{
    size_t count;
    char** properties;
} js_object_properties_t;

typedef const void* js_class_template_t;

typedef enum class_definition_type_e
{
    JS_CLASS,
    JS_FUNCTION
} class_definition_type_t;

typedef enum class_definition_iframe_behaviour_e
{
    //object should not be initalized in iframes
    //it is default one
    NONE,
    //object should be copied as reference to each iframe
    REFERENCE,
    //object should be created for each iframe and NOT inform plugin
    CREATE_INSTANCE,
} class_definition_iframe_behaviour_t;

typedef enum class_definition_iframe_notice_e
{
    //it is default one
    NONE_NOTICE,
    ALWAYS_NOTICE
} class_definition_iframe_notice_t;

typedef enum class_definition_iframe_overlay_e
{
    IGNORED,
    USE_OVERLAYED,
    OVERLAYED_BEFORE_ORIGINAL
} class_definition_iframe_overlay_t;

typedef void* js_object_instance_t;
//global_context - id
typedef void(*iframe_loaded_cb)(java_script_context_t global_context,
                                js_object_instance_t iframe,
                                js_object_instance_t object);

typedef struct class_definition_options_s
{
    class_definition_type_t type;
    class_definition_iframe_behaviour_t iframe_option;
    class_definition_iframe_notice_t iframe_notice;
    class_definition_iframe_overlay_t iframe_overlay;
    iframe_loaded_cb cb;
    void * private_data;
} class_definition_options_t;


/*
 * list of device caps
 */
typedef struct devcaps_s
{
    char** deviceCaps;
    size_t devCapsCount;
} devcaps_t;

/*
 * mapping from a feature to corresponding list of device capabilities
 */
typedef struct feature_devcaps_s
{
    char* feature_name;
    devcaps_t devCaps;
} feature_devcaps_t;

/*
 * list of feature_devcaps_t structs
 */
typedef struct feature_mapping_s
{
    feature_devcaps_t* features;
    size_t featuresCount;
} feature_mapping_t;

/*
 * This is a structure describing a JS class templated, object name and
 * it's parent class name (parent_name)
 * JS class will be bind to a parent class name (parent_name.jsclass_name).
 * @param parent_name - parent name (ie Widget.Device)
 * @param object_name - object name (DeviceStatus)
 * @param js_class_template - js_class_template required to create JS object
 * @param private_data private data for object creator if required
 * (usually NULL)
 */
typedef struct class_definition_s
{
    const char *parent_name;
    const char *object_name;
    js_class_template_t js_class_template;
    //class options may be null - default
    class_definition_options_t* class_options;
} class_definition_t;

/**
 * FIXME: Add documentation
 */
typedef const class_definition_t *class_definition_ptr_t;

/**
 * Plugin export names
 */
#define PLUGIN_WIDGET_START_PROC        on_widget_start
#define PLUGIN_WIDGET_STOP_PROC         on_widget_stop
#define PLUGIN_CLASS_MAP                class_map
#define PLUGIN_GET_CLASS_PROC_MAP       get_widget_class_map

#define PLUGIN_WIDGET_START_PROC_NAME   "on_widget_start"
#define PLUGIN_WIDGET_STOP_PROC_NAME    "on_widget_stop"
#define PLUGIN_CLASS_MAP_NAME           "class_map"
#define PLUGIN_GET_CLASS_MAP_PROC_NAME  "get_widget_class_map"

/**
 * Plugin export typedefs
 */
typedef void (*on_widget_start_proc)(int widgetId);

/**
 * FIXME: Add documentation
 */
typedef void (*on_widget_stop_proc)(int widgetId);

typedef const class_definition_t* (*get_widget_class_map_proc)();

#endif // WRT_PLUGIN_EXPORT_H
